"""
Copyright 2019 de la Dirección General de Sistemas Informáticos – Secretaría Legal y Técnica - Nación.

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see http://www.gnu.org/licenses/
"""
from django.core.checks import Warning, register, Tags as DjangoTags

from dapp.settings import NODE_URL, IS_POA, CONTRACT_ADDRESS, BYTECODE, ACCOUNT
from gateway.gateway import generate_http_gateway, NoConnectionException


class Tags(DjangoTags):
    """Do this if none of the existing tags work for you:
    https://docs.djangoproject.com/en/1.8/ref/checks/#builtin-tags
    """
    my_new_tag = 'dapp'


def check_geth(app_configs, **kwargs):
    errors = []
    try:
        gw = generate_http_gateway(NODE_URL, IS_POA)
        bytecode = gw.get_smart_contract_bytecode(CONTRACT_ADDRESS)
        if bytecode is not BYTECODE:
            errors.append(Warning(
                'Bytecode to {} does not match with BYTECODE setting'.format(CONTRACT_ADDRESS),
                hint='You should redeploy the smart contract or set the address correctly',
                obj=NODE_URL,
                id='dapp.W001', ))
        if gw.get_balance(ACCOUNT) == 0:
            errors.append(Warning(
                'Balance of ACCOUNT setting {} is zero'.format(ACCOUNT),
                hint='You should redeploy the smart contract or set the address correctly',
                obj=NODE_URL,
                id='dapp.W001', ))
    except NoConnectionException as nce:
        errors.append(Warning(
            'No connection to {}'.format(NODE_URL),
            hint='You should check if connection is available to the geth node',
            obj=NODE_URL,
            id='dapp.W001', ))

    return errors
