"""
Copyright 2019 de la Dirección General de Sistemas Informáticos – Secretaría Legal y Técnica - Nación.

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see http://www.gnu.org/licenses/
"""
from django.core.management.base import BaseCommand
import logging

from gateway.gateway import ContractDeployer, Gateway, HTTPProviderGenerator

from dapp.settings import *

logger = logging.getLogger('cmd-logger')


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--gas', help='Gas a usar', type=int)
        parser.add_argument('--wait', help="Wait for receipt", action='store_true')
        parser.set_defaults(gas=0, wait=False)

    def handle(self, *args, **options):
        try:
            provider = HTTPProviderGenerator(NODE_URL)
            gateway = Gateway(provider, IS_POA)
            self.stdout.write(str(gateway.get_node_accounts()))
            unlocked = gateway.unlock_account(ACCOUNT, PASSPHRASE)
            if not unlocked:
                raise Exception("Contraseña incorrecta")
            deployer = ContractDeployer(gateway)
            gas = deployer.estimate_deploy(ABI, BYTECODE)
            logger.info("Gas estimado: {}".format(str(gas)))
            if options['gas'] is not 0:
                gas_aux = options['gas']
            else:
                gas_aux = input("Ingrese un límite de gas opcional: ")
            if gas_aux is not '':
                gas = int(gas_aux)

            tx_receipt = deployer.deploy_contract(ABI, BYTECODE, gas, ACCOUNT, options['wait'])
            logger.info("Contract address: {}".format(str(tx_receipt['contractAddress'])))
            logger.info("tx_receipt: {}".format(str(tx_receipt)))
        except Exception as e:
            logger.exception(e)
